import json
from typing import Any, Optional, Set
import datetime
import fsspec
import s3fs
from fsspec import AbstractFileSystem
from fsspec.implementations.local import LocalFileSystem
from pymongo import MongoClient

from sziszapangma.integration.repository.experiment_repository import ExperimentRepository
from sziszapangma.integration.repository.mongo_experiment_repository import MongoExperimentRepository
from sziszapangma.integration.repository.multi_files_experiment_repository import MultiFilesExperimentRepository


class FsspecExperimentRepository(ExperimentRepository):
    _fsspec: AbstractFileSystem
    _dataset_name: str
    _pretty_format: bool

    def __init__(self, file_system: AbstractFileSystem, dataset_name: str, pretty_format: bool = False):
        super().__init__()
        self._fsspec = file_system
        self._dataset_name = dataset_name
        self._pretty_format = pretty_format

    def initialise(self) -> None:
        return

    def property_exists(self, record_id: str, property_name: str) -> bool:
        path = self._get_path_for(record_id, property_name)
        return self._fsspec.exists(path)

    def update_property_for_key(self, record_id: str, property_name: str, property_value: Any):
        path = self._get_path_for(record_id, property_name)
        json.dump(property_value, self._fsspec.open(path, 'w'))

    def delete_property_for_key(self, record_id: str, property_name: str):
        path = self._get_path_for(record_id, property_name)
        self._fsspec.delete(path)

    def get_property_for_key(self, record_id: str, property_name: str) -> Optional[Any]:
        path = self._get_path_for(record_id, property_name)
        if self._fsspec.exists(path):
            return json.load(self._fsspec.open(path, 'r'))

    def get_all_record_ids(self) -> Set[str]:
        all_records_set = set()
        for property_name in self.get_all_properties():
            all_records_set.update(self.get_all_records_for_property(property_name))
        return all_records_set

    def get_all_records_for_property(self, property_name: str) -> Set[str]:
        path = self._get_property_path(property_name)
        return set([it.split('/')[-1][:-5] for it in self._fsspec.ls(path)])

    def get_all_properties(self) -> Set[str]:
        path = self._get_dataset_path()
        return set([it.split('/')[-1] for it in self._fsspec.ls(path)])

    def _get_path_for(self, record_id: str, property_name: str) -> str:
        return f'{self._get_property_path(property_name)}/{record_id}.json'

    def _get_dataset_path(self) -> str:
        return f'/pipeline/{self._dataset_name}'

    def _get_property_path(self, property_name: str) -> str:
        return f'{self._get_dataset_path()}/{property_name}'


def copy_repos(from_repo: ExperimentRepository, to_repo: ExperimentRepository, dataset_name: str):
    all_ids = list(from_repo.get_all_record_ids())
    for prop in sorted(list(from_repo.get_all_properties())):
        print(datetime.datetime.now().__str__(), f'{dataset_name} -> {prop}')
        for record_index in range(len(all_ids)):
            record = all_ids[record_index]
            if from_repo.property_exists(record, prop):
                to_repo.update_property_for_key(record, prop, from_repo.get_property_for_key(record, prop))


def run_migration_for_repo(dataset_name: str):
    mongo = MongoClient('mongodb://root:example@mongo-asr-benchmarks.theliver.pl:27021/', ssl=True)
    mongo_repo = MongoExperimentRepository(mongo_client=mongo, database_name=dataset_name)
    repo_local = MultiFilesExperimentRepository('/home/marcinwatroba/PWR_ASR/asr-benchmarks/experiment_data/pipeline',
                                                dataset_name)
    copy_repos(repo_local, mongo_repo, dataset_name)


# if __name__ == '__main__':
#     print(MongoClient('mongodb://root:example@mongo-asr-benchmarks.theliver.pl:27021/', ssl=True))
#     datasets = ['pl_google_fleurs', 'pl_luna', 'pl_minds14', 'pl_voicelab_cbiz', 'pl_common_voice']
#     for it in datasets:
#         run_migration_for_repo(it)
