import io
import json
from typing import Any, List, Optional, Set

from minio import Minio

from sziszapangma.integration.repository.experiment_repository import ExperimentRepository


class MinioExperimentRepository(ExperimentRepository):
    _bucket_name: str
    _root_path: str
    _experiment_name: str
    _client: Minio

    def __init__(self, client: Minio, bucket_name: str, root_path: str, experiment_name: str):
        self._client = client
        self._bucket_name = bucket_name
        self._root_path = root_path
        self._experiment_name = experiment_name

    def initialise(self) -> None:
        return

    def property_exists(self, record_id: str, property_name: str) -> bool:
        path = self._get_file_path(property_name, record_id)
        return self._client.get_object(self._bucket_name, path).status < 400

    def update_property_for_key(self, record_id: str, property_name: str, property_value: Any):
        path = self._get_file_path(property_name, record_id)
        content_bytes = json.dumps(property_value).encode("utf-8")
        print(self._bucket_name, path)
        self._client.put_object(
            self._bucket_name, path, io.BytesIO(content_bytes), len(content_bytes)
        )

    def delete_property_for_key(self, record_id: str, property_name: str):
        path = self._get_file_path(property_name, record_id)
        self._client.remove_object(self._bucket_name, path)

    def get_property_for_key(self, record_id: str, property_name: str) -> Optional[Any]:
        if self.property_exists(record_id, property_name):
            path = self._get_file_path(property_name, record_id)
            json_content = self._client.get_object(self._bucket_name, path).read().decode("utf-8")
            return json.loads(json_content)
        else:
            return None

    def get_all_record_ids(self) -> Set[str]:
        ids: List[str] = []
        for property_name in self.get_all_properties():
            path = f"{self._root_path}{self._experiment_name}/{property_name}/"
            property_ids = set(
                [
                    obj.object_name.split("/")[-1].replace(".json", "")
                    for obj in self._client.list_objects(self._bucket_name, path)
                ]
            )
            ids.extend(property_ids)
        return set(ids)

    def get_all_properties(self) -> Set[str]:
        experiment_path = f"{self._root_path}{self._experiment_name}/"
        return set(
            [
                obj.object_name.split("/")[-1]
                for obj in self._client.list_objects(self._bucket_name, experiment_path)
            ]
        )

    def _get_file_path(self, property_name: str, record_id: str) -> str:
        return f"{self._root_path}{self._experiment_name}/{property_name}/{record_id}.json"
